package it.eng.spago.services.fop;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.presentation.StylesheetFile;
import it.eng.spago.services.AbstractService;
import it.eng.spago.tracing.TracerSingleton;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.Hashtable;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.fop.apps.Driver;
import org.apache.fop.messaging.MessageHandler;

/**
 * Service that convert an xml stream into other formats using
 * <a href="http://xmlgraphics.apache.org/fop/index.html">FOP</a>
 * (an xsl-fo processor provided by apache).
 * <br/>
 * NOTE: the version used is the 0.20.5
 * 
 * @author buso
 *
 */
public class FOPService extends AbstractService implements Serializable {
	private static final long serialVersionUID = 1L; 
	
	/**
	 * Execute an xsl transformation to obtain an xsl-fo document from the
	 * <code>xml stream</code> passed as parameter. The obtained xsl-fo is
	 * processed by FOP to be rendered in the <code>renderType</code> format
	 * passed as parameter.
	 * 
	 * @param xml. The <code>xml stream</code> containing the content to be rendered
	 * 		with FOP.
	 * @param xslName. Name of the <code>xsl</code> configured in publishers.xml
	 * 		used to transform the <code>xml stream</code> passed to xsl-fo. 
	 * @param renderType. Has to be one of the render type supported
	 * by <a href="http://xmlgraphics.apache.org/fop/" a>FOP</a> defined
	 * in @see {@link org.apache.fop.apps.Driver} 
	 * @param out. Stream where FOP will render the document.
	 * @throws TransformerException if error occure during xsl transformation
	 * or FO processing.
	 */
	public void process(InputStream xml, String xslName,
		int renderType, OutputStream out, RequestContextIFace requestCtx)
		throws TransformerException {

        if (xml == null) {
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
        		"FOPService::process: passed xml is null.");
        	return;
        }
        
		ApplicationContainer application = ApplicationContainer.getInstance();
		Hashtable xslTemplates = (Hashtable) application
			.getAttribute("XSL_TEMPLATES");
		if (xslTemplates == null) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"FOPService::process: xslTemplates nullo");
			return ;
		} // if (xslTemplates == null)
		// Retrive from cache the file identified by the label xslName
		StylesheetFile stylesheet = (StylesheetFile) xslTemplates
			.get(xslName);
		if (stylesheet == null) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"FOPService::process: template di stylesheetName ["
							+ xslName + "] non trovato");
			return;
		} // if (stylesheet == null)
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"FOPService::process: template di stylesheetName ["
				+ xslName + "]");
		// Retrive the template rappresenting the processed xsl transformation
		Templates template = stylesheet.getTemplate();
		try {
			Transformer transformer = template.newTransformer();
			Source src = new StreamSource(xml);
			// set FOP driver
			Driver driver = new Driver();
			MessageHandler.setQuiet(true);
			driver.setRenderer(renderType);
			driver.setOutputStream(out);
			
			Result res = new SAXResult(driver.getContentHandler());
			transformer.transform(src, res);
		} //try
		catch (TransformerConfigurationException e) {
//			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
//					"FOPService::process: ", e);
			throw e;
		} catch (TransformerException e) {
//			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
//					"FOPService::process: ", e);
			throw e;
		}
	}
}
